# Generated by Django 4.2.4 on 2023-08-29 12:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0002_alter_project_slug"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="email_sender_id",
            field=models.CharField(max_length=320, null=True),
        ),
        migrations.AddField(
            model_name="project",
            name="sms_sender_id",
            field=models.CharField(max_length=11, null=True),
        ),
    ]
