# Generated by Django 4.2.4 on 2023-10-26 06:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0009_projectsettings_pending_value"),
    ]

    operations = [
        migrations.AlterField(
            model_name="projectsettings",
            name="action_required",
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
        migrations.AlterField(
            model_name="projectsettings",
            name="pending_value",
            field=models.CharField(blank=True, max_length=320, null=True, verbose_name="Pending value"),
        ),
    ]
