# Generated by Django 4.2.4 on 2023-09-20 08:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0005_remove_project_email_sender_id_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="projectsettings",
            name="value_type",
            field=models.CharField(
                choices=[
                    ("integer", "Whole number"),
                    ("float", "Decimal number"),
                    ("bool", "True/False"),
                    ("char", "String"),
                    ("custom", "Custom"),
                ],
                max_length=10,
            ),
        ),
    ]
