# Generated by Django 4.2.4 on 2023-09-01 10:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0003_project_email_sender_id_project_sms_sender_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="demoprojecttag",
            name="name",
            field=models.CharField(max_length=100, verbose_name="name"),
        ),
        migrations.AlterField(
            model_name="demoprojecttag",
            name="slug",
            field=models.SlugField(allow_unicode=True, max_length=100, verbose_name="slug"),
        ),
        migrations.AlterUniqueTogether(
            name="demoprojecttag",
            unique_together={("project", "slug"), ("project", "name")},
        ),
    ]
