# Generated by Django 4.2.4 on 2023-09-27 05:54

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0006_alter_projectsettings_value_type"),
        ("demo_django_base", "0006_projectsettings_reserved_and_more"),
    ]

    operations = []
